/*
    FreeRTOS V6.0.5 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*-----------------------------------------------------------
 * Characters on the LCD are used to simulate LED's.  In this case the 'ParTest'
 * is really operating on the LCD display.
 *-----------------------------------------------------------*/

/*
 * This demo is configured to execute on the IAR LPC1114 Evaluation board. This
 * board has a built in LCD display and two user accessible LEDs. Therefore, in 
 * place of flashing an LED, the 'flash' and 'check' tasks toggle '*' characters
 * on the LCD.  The left most '*' represents LED 0, the next LED 1, etc.
 *
 * The two genuine on board LEDs referenced as LED 10 and LED 11.
 */

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo application includes. */
#include "partest.h"
#include "drv_hd44780.h"

/* Constants required to access the "LED's".  The LED segments are turned on
and off to generate '*' characters. */
#define partstNUM_LEDS			( ( unsigned portCHAR ) 6 )

/* The LED number of the real on board LED, rather than a simulated LED. */
#define partstON_BOARD_LED		( ( unsigned portBASE_TYPE ) 10 )

/* Storage for current LED state - used by toggle routine */
static char acLEDState[partstNUM_LEDS];
/*
 * Toggle the single genuine built in LED.
 */
static void prvToggleOnBoardLED( void );

/*-----------------------------------------------------------*/

void vParTestInitialise( void )
{
    /* Initialise IO used to drive the onboard LED */
    LPC_IOCON->PIO1_9 = 0;           /* Set as DIO no PU or PD resistor */
    LPC_GPIO1->DIR   |= (1UL << 9);  /* Set as output */
    LPC_GPIO1->DATA  |= (1UL << 9);  /* Turn LED off */

    /* Initialise the LCD hardware. */
    HD44780_IO_Init();
    HD44780_PowerUpInit();       
}
/*-----------------------------------------------------------*/

void vParTestSetLED( unsigned portBASE_TYPE uxLED, signed portBASE_TYPE xValue )
{
    /* Set or clear the output [in this case show or hide the '*' character. */
    if( uxLED < ( portBASE_TYPE ) partstNUM_LEDS )
    {
        vTaskSuspendAll();
	{
	    if( xValue )
	    {
	        /* Turn on the segments required to show the '*'. */
                HD44780_StrShow(uxLED+1,1,(const HD44780_STRING_DEF *) "*");            
            }
	    else
	    {
		/* Turn off the segments. */
                HD44780_StrShow(uxLED+1,1,(const HD44780_STRING_DEF *) " ");            
            }
	}
	xTaskResumeAll();
    }
}
/*-----------------------------------------------------------*/

void vParTestToggleLED( unsigned portBASE_TYPE uxLED )
{
    if( uxLED < ( portBASE_TYPE ) partstNUM_LEDS )
    {
        vTaskSuspendAll();
	{
	    /* If the '*' is already showing - hide it.  If it is not already
	       showing then show it. */
       	    if( acLEDState[uxLED] == ' ' )
	    {
	        /* Turn on the segments required to show the '*'. */
                HD44780_StrShow(uxLED+1,1,(const HD44780_STRING_DEF *) "*");            
                acLEDState[uxLED] = '*';
            }
	    else
	    {
		/* Turn off the segments. */
                HD44780_StrShow(uxLED+1,1,(const HD44780_STRING_DEF *) " ");            
                acLEDState[uxLED] = ' ';                
            }
        }
	xTaskResumeAll();
    }
    else
    {
        if( uxLED == partstON_BOARD_LED )
	{
	    /* The request related to the genuine on board LED. */
	    prvToggleOnBoardLED();
	}
    }	
}
/*-----------------------------------------------------------*/

static void prvToggleOnBoardLED( void )
{
    static unsigned portSHORT sState = pdFALSE;

    /* Toggle the state of the single genuine on board LED. */
    if( sState )	
    {
        LPC_GPIO1->DATA &= ~(1UL << 9); /* Turn LED on*/
    }
    else
    {
        LPC_GPIO1->DATA |= (1UL << 9); /* Turn LED off */
    }
    sState = !sState;
}
/*-----------------------------------------------------------*/


