/*
    FreeRTOS V6.0.5 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/


/* BASIC INTERRUPT DRIVEN SERIAL PORT DRIVER.
 *
 * This file only supports UART 1
 */

/* Standard includes. */
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

/* Demo application includes. */
#include "LPC11xx.h"
#include "serial.h"

/* UART register bit definitons */
#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80

/* Misc. constants. */
#define serNO_BLOCK	( ( portTickType ) 0 )

/* The queue used to hold received characters. */
static xQueueHandle xRxedChars;

/* The queue used to hold characters waiting transmission. */
static xQueueHandle xCharsForTx;

static volatile unsigned int uxTHREEmpty = pdFALSE;

/*-----------------------------------------------------------*/

xComPortHandle xSerialPortInitMinimal( unsigned portLONG ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
{
  uint32_t Fdiv;
  uint32_t regVal;

  portENTER_CRITICAL();
  {
    /* Create the queues used by the com test task. */
    xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
    xCharsForTx = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );

    /* Initialise the THRE empty flag. */
    uxTHREEmpty = pdTRUE;
    
    NVIC_DisableIRQ(UART_IRQn);

    LPC_IOCON->PIO1_6 &= ~0x07;    /* UART I/O config */
    LPC_IOCON->PIO1_6 |= 0x01;     /* UART RXD */
    LPC_IOCON->PIO1_7 &= ~0x07;	
    LPC_IOCON->PIO1_7 |= 0x01;     /* UART TXD */

    /* Enable UART clock */
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
    LPC_SYSCON->UARTCLKDIV = 0x1;     /* divided by 1 */

    LPC_UART->LCR = 0x83;             /* 8 bits, no Parity, 1 Stop bit */
    regVal = LPC_SYSCON->UARTCLKDIV;
    Fdiv = ((SystemCoreClock/regVal)/16)/ulWantedBaud ;	/*baud rate */

    LPC_UART->DLM = Fdiv / 256;							
    LPC_UART->DLL = Fdiv % 256;
    LPC_UART->LCR = 0x03;		/* DLAB = 0 */
    LPC_UART->FCR = 0x07;		/* Enable and reset TX and RX FIFO. */

    /* Read to clear the line status. */
    regVal = LPC_UART->LSR;

    /* Ensure a clean start, no data in either TX or RX FIFO. */
    while ( LPC_UART->LSR & (LSR_THRE|LSR_TEMT) != (LSR_THRE|LSR_TEMT) );
    while ( LPC_UART->LSR & LSR_RDR )
    {
      regVal = LPC_UART->RBR;	/* Dump data from RX FIFO */
    }

    /* Enable the UART Interrupt */
    NVIC_EnableIRQ(UART_IRQn);
    LPC_UART->IER = IER_RBR | IER_THRE | IER_RLS;
  }
  portEXIT_CRITICAL();
	
  /* Unlike other ports, this serial code does not allow for more than one
     com port.  We therefore don't return a pointer to a port structure and can
     instead just return NULL. */
  return NULL;
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialGetChar( xComPortHandle pxPort, signed portCHAR *pcRxedChar, portTickType xBlockTime )
{
  /* The port handle is not required as this driver only supports UART0. */
  ( void ) pxPort;
  
  /* Get the next character from the buffer.  Return false if no characters
     are available, or arrive before xBlockTime expires. */
  if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
  {
    return pdTRUE;
  }
  else
  {
    return pdFALSE;
  }
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialPutChar( xComPortHandle pxPort, signed portCHAR cOutChar, portTickType xBlockTime )
{
  signed portBASE_TYPE xReturn;

  /* Transmit a character. */
  portENTER_CRITICAL();
  {  
    /* Is there space to write directly to the UART? */
    if( uxTHREEmpty == ( unsigned int ) pdTRUE )
    {
      /* If uxTHREEmpty flag is true then the UART Tx ISR has indicated that
	 there are no characters queued to be transmitted - so we can
	 write the character directly to the shift Tx register. */
      LPC_UART->THR = cOutChar;
      uxTHREEmpty = pdFALSE;
      xReturn = pdPASS;
    }
    else
    {
      /* THRE bit is false, so there are still characters waiting to be
	 transmitted.  We have to queue this character so it gets
	 transmitted in turn. */

      /* Return false if after the block time there is no room on the Tx
	 queue. It is ok to block inside a critical section as each task
	 maintains it's own critical section status. */
      xReturn = xQueueSend( xCharsForTx, &cOutChar, xBlockTime );
                        
      /* Depending on queue sizing and task prioritisation:  While we
	 were blocked waiting to post on the queue interrupts were not
	 disabled. It is possible that the serial ISR has emptied the
	 Tx queue, in which case we need to start the Tx off again
	 writing directly to the Tx register. */
      if( ( uxTHREEmpty == ( unsigned int ) pdTRUE ) && ( xReturn == pdPASS ) )
      {
        /* Get back the character we just posted. */
	xQueueReceive( xCharsForTx, &cOutChar, serNO_BLOCK );
        uxTHREEmpty = pdFALSE;
        LPC_UART->THR = cOutChar;
      }
    }
  }
  portEXIT_CRITICAL();
  
  return pdPASS;
}

/*-----------------------------------------------------------*/

void UART0_IRQHandler(void)
{  
  signed portCHAR cChar;
  portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
  volatile unsigned int uxIIR, uxLSR;

  /* Read UART interrupt status register to clear the bits */
  uxIIR   = LPC_UART->IIR;
  uxIIR >>= 1;		    /* skip pending bit in IIR */
  uxIIR  &= 0x07;	    /* check bit 1~3, interrupt identification */

  if (uxIIR == IIR_RLS)	    /* Receive Line Status */
  {
    uxLSR = LPC_UART->LSR;

    /* Receive Line Status */
    if (uxLSR & (LSR_OE | LSR_PE | LSR_FE | LSR_RXFE | LSR_BI))
    {
      /* There are errors or break interrupt. Dummy read on RX to clear 
         interrupt, then bail out */
      volatile char cDummy = LPC_UART->RBR;	
      return;
    }
    /* Receive Data Ready */			
    if (uxLSR & LSR_RDR)	
    {
      /* Get the character from the UART and post it on the queue of Rxed
    	 characters. */
      cChar = LPC_UART->RBR;
	
      xQueueSendFromISR( xRxedChars, &cChar, &xHigherPriorityTaskWoken );

      if( xHigherPriorityTaskWoken )
      {
        /* If the post causes a task to wake force a context switch
	   as the woken task may have a higher priority than the task we have
	   interrupted. */
        taskYIELD();
      }
    }
  }
  else if (uxIIR == IIR_RDA)	/* Receive Data Available */
  {	
    /* Get the character from the UART and post it on the queue of Rxed
       characters. */
    cChar = LPC_UART->RBR;
	
    xQueueSendFromISR( xRxedChars, &cChar, &xHigherPriorityTaskWoken );

    if( xHigherPriorityTaskWoken )
    {
      /* If the post causes a task to wake force a context switch
	 as the woken task may have a higher priority than the task we have
	 interrupted. */
      taskYIELD();
    }
  }
  else if (uxIIR == IIR_THRE)	/* THRE, transmit holding register empty */
  {
    /* The previous character has been transmitted.  See if there are any
       further characters waiting transmission. */	
    if( xQueueReceiveFromISR( xCharsForTx, &cChar, &xHigherPriorityTaskWoken ) == pdTRUE )
    {
      /* There was another character queued - transmit it now. */
      LPC_UART->THR = cChar;
    }
    else
    {
      /* There are no further characters queued to send so we can indicate
         that the THRE is available. */
      uxTHREEmpty = pdTRUE;
    }
  }     
  portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );  
}
/*-----------------------------------------------------------*/
