/*
    FreeRTOS V6.0.5 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*-----------------------------------------------------------
 * Implementation of functions defined in portable.h for the ARM CM0 port.
 *----------------------------------------------------------*/

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"

/* For backward compatibility, ensure configKERNEL_INTERRUPT_PRIORITY is
defined.  The value should also ensure backward compatibility.
FreeRTOS.org versions prior to V4.4.0 did not include this definition. */
#ifndef configKERNEL_INTERRUPT_PRIORITY
  #define configKERNEL_INTERRUPT_PRIORITY 3
#endif

/* Constants required to manipulate the NVIC. */
#define portNVIC_PENDSVSET      0x10000000

/* Constants required to set up the initial stack. */
#define portINITIAL_XPSR      ( 0x01000000 )

/* Each task maintains its own interrupt status in the critical nesting
variable. */
static volatile unsigned portBASE_TYPE uxCriticalNesting = 0xaaaaaaaa;

/*
 * Setup the timer to generate the tick interrupts.
 */
static void prvSetupTimerInterrupt( void );

/*
 * Exception handlers.
 */
void xPortSysTickHandler( void );

/*
 * Start first task is a separate function so it can be tested in isolation.
 */
extern void vPortStartFirstTask( void );

/*-----------------------------------------------------------*/

/*
 * See header file for description.
 */
portSTACK_TYPE *pxPortInitialiseStack( portSTACK_TYPE *pxTopOfStack, pdTASK_CODE pxCode, void *pvParameters )
{
  /* Simulate the stack frame as it would be created by a context switch
  interrupt. */
  *pxTopOfStack = portINITIAL_XPSR;                /* xPSR */
  pxTopOfStack--;
  *pxTopOfStack = ( portSTACK_TYPE ) pxCode;       /* Return address (new PC) */
  pxTopOfStack--;
  *pxTopOfStack = 0;                               /* LR (R14) */
  pxTopOfStack -= 5;                               /* R12, R3, R2, R1 */
  *pxTopOfStack = ( portSTACK_TYPE ) pvParameters; /* and R0 */
  pxTopOfStack -= 8;                               /* R7, R6, R5, R4, R11, R10, R9, and R8. */
    
  return pxTopOfStack;
}
/*-----------------------------------------------------------*/

/*
 * See header file for description.
 */
portBASE_TYPE xPortStartScheduler( void )
{
  /* Make PendSV, CallSV and SysTick the same priroity as the kernel. */
  /* set Priority for Cortex-M0 System Interrupts */
  NVIC_SetPriority (SysTick_IRQn, configKERNEL_INTERRUPT_PRIORITY);                            
  NVIC_SetPriority (PendSV_IRQn, configKERNEL_INTERRUPT_PRIORITY);

  /* Start the timer that generates the tick ISR.  Interrupts are disabled
  here already. */
  prvSetupTimerInterrupt();

  /* Initialise the critical nesting count ready for the first task. */
  uxCriticalNesting = 0;

  /* Needed here since CM0 doesn't have interrupt priority masking */
  portENABLE_INTERRUPTS();

  /* Start the first task. */
  vPortStartFirstTask();

  /* Should not get here! */
  return 0;
}
/*-----------------------------------------------------------*/

void vPortEndScheduler( void )
{
  /* It is unlikely that the CM0 port will require this function as there
  is nothing to return to.  */
}
/*-----------------------------------------------------------*/

void vPortYieldFromISR( void )
{
  /* Set a PendSV to request a context switch. */
  SCB->ICSR = portNVIC_PENDSVSET;
}
/*-----------------------------------------------------------*/

void vPortEnterCritical( void )
{
  portDISABLE_INTERRUPTS();
  uxCriticalNesting++;
}
/*-----------------------------------------------------------*/

void vPortExitCritical( void )
{
  uxCriticalNesting--;
  if( uxCriticalNesting == 0 )
  {
    portENABLE_INTERRUPTS();
  }
}
/*-----------------------------------------------------------*/

void xPortSysTickHandler( void )
{
unsigned portLONG ulDummy;

  /* If using preemption, also force a context switch. */
  #if configUSE_PREEMPTION == 1
    SCB->ICSR = portNVIC_PENDSVSET;
  #endif

  ulDummy = portSET_INTERRUPT_MASK_FROM_ISR();
  {
    vTaskIncrementTick();
  }
  portCLEAR_INTERRUPT_MASK_FROM_ISR( ulDummy );
}
/*-----------------------------------------------------------*/

/*
 * Setup the systick timer to generate the tick interrupts at the required
 * frequency.
 */
void prvSetupTimerInterrupt( void )
{
  /* Configure SysTick to interrupt at the requested rate. */
  SysTick_Config( configCPU_CLOCK_HZ / configTICK_RATE_HZ );
}
/*-----------------------------------------------------------*/

